--[[ $Id: Options.lua 78023 2008-07-07 21:57:15Z ag $ ]]--
local LibStub = LibStub
local Fortress = LibStub("AceAddon-3.0"):GetAddon("Fortress")

--local L = LibStub("AceLocale-3.0"):GetLocale("Fortress")
local L = newproxy(true)
getmetatable(L).__index = function(self, key)
	return key
end

local function debug(...)
-- local tmp = {"Fortress: ", ...}
-- ChatFrame1:AddMessage(table.concat(tmp, " "))
end

local db
local appName = "Fortress"

--[[ Get/Set functions ]]--
local function DefaultGet(info)
	return db[info.arg or info[#info]] 
end

local function DefaultSet(info, value)
	db[info.arg or info[#info]] = value
end

local function PluginGet(info)
	local name = info.appName
	return db.pluginDB[name][info.arg or info[#info]]
end

local function PluginSet(info, value)
	local name = info.appName
	db.pluginDB[name][info.arg or info[#info]] = value
	Fortress:UpdateObject(name)
end

local function PluginColorGet(info)
	local name = info.appName
	local clr = db.pluginDB[name][info[#info]]
	return clr.r, clr.g, clr.b, clr.a
end

local function PluginColorSet(info, r, g, b, a)
	local name = info.appName
	local clr = db.pluginDB[name][info[#info]]
	clr.r = r
	clr.g = g
	clr.b = b
	clr.a = a
	Fortress:UpdateColor(name)
end

local function PluginBlockGet(info)
	local name = info.appName
	return db.blockDB[name][info.arg or info[#info]]
end

local function PluginBlockSet(info, value)
	local name = info.appName
	db.blockDB[name][info.arg or info[#info]] = value
end

local function PluginDisabled(info)
	local name = info.appName
	return not db.pluginDB[name].enabled
end

--[[ options table ]]--
local options = {
	handler = Fortress,
	get     = DefaultGet,
	set     = DefaultSet,
	name    = L["Fortress"],
	type    = "group",
	childGroups = "tab",
	args    = {
		general = {
			name = L["General"],
			desc = L["General options."],
			type = "group",
			args = {
				enabled = {
					name = L["Enabled"],
					desc = L["Enable/disable the addon."],
					type = "toggle",
					get  = "IsEnabled",
					set  = function(info, value)
						DefaultSet(info, value)
						if value then
							Fortress:Enable()
						else
							Fortress:Disable()
						end
					end,
				},
				hideInCombat = {
					name = L["Hide in combat"],
					desc = L["Hides all plugins in combat."],
					type = "toggle",
				},
				hideTTInCombat = {
					name = L["Hide tooltip in combat"],
					desc = L["Hides the default tooltip for plugins in combat."],
					type = "toggle",
				},
			},
		},
--		plugins = {
--			name = L["Plugins"],
--			desc = L["Individual plugin settings."],
--			type = "group",
--			args = {	-- args added with :AddObjectOptions
--			},
--		},
	},
}

local pluginOptions = {
	group1 = {
		name = "",
		inline = true,
		type = "group",
		order = 1,
		args = {
			enabled = {
				name = L["Enabled"],
				desc = L["Enables/disables the plugin."],
				type = "toggle",
				get  = PluginGet,
				set  = function(info, value)
					local name = info.appName
					if value then
						Fortress:EnableDataObject(name)
					else
						Fortress:DisableDataObject(name)
					end
				end,
			},
			locked = {
				name = L["Locked"],
				desc = L["Lock/unlock the plugin."],
				type = "toggle",
				get  = PluginBlockGet,
				set  = PluginBlockSet,
				disabled = PluginDisabled,
			},
			showLabel = {
				name = L["Show Label"],
				desc = L["Shows the object label."],
				type = "toggle",
				get  = PluginGet,
				set  = PluginSet,
				disabled = PluginDisabled,
			},			
			showIcon = {
				name = L["Show Icon"],
				desc = L["Show the plugins icon, if available."],
				type = "toggle",
				get  = PluginGet,
				set  = PluginSet,
				disabled = function(info)
					return PluginDisabled(info) or not db.pluginDB[info.appName].hasIcon
				end
			},
			hideInCombat = {
				name = L["Hide in combat"],
				desc = L["Hide this plugin when you enter combat."],
				type = "toggle",
				get  = PluginGet,
				set  = PluginSet,
				disabled = function(info)
					return PluginDisabled(info) or db.hideInCombat
				end,
			},
			borderColor	= {
				name = L["Borders"],
				type = "color",
				desc = L["Change the frame border color."],
				hasAlpha = true,
				get  = PluginColorGet,
				set  = PluginColorSet,
			},
			frameColor	= {
				name = L["Background"],
				type = "color",
				desc = L["Change the frame background color."],
				hasAlpha = true,
				get  = PluginColorGet,
				set  = PluginColorSet,		
			},
		}
	},
	group2 = {
		name = "",
		inline = true,
		type = "group",
		order = 2,
		args = {
			fontSize = {
				name = L["Font size"],
				desc = L["Sets the font size of the display."],
				type = "range",
				get  = PluginGet,
				set  = PluginSet,
				min = 5,
				max = 20,
				step = 1,
			},	
		}
	}
}

local ChatCmd
do
	local AceCfgDlg = LibStub("AceConfigDialog-3.0")
	local AceCfgCmd = LibStub("AceConfigCmd-3.0")

	function ChatCmd(input)
		if not input or input:trim() == "" then
			AceCfgDlg:Open(appName)
		else
			AceCfgCmd.HandleCommand(self, "fortress", appName, input)
		end
	end
end

function Fortress:RegisterOptions()
	db = assert(self.db.profile)	-- assert to ensure that AceDB:New() gets called before this function
	self.optionsFrames = {}
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(appName, options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(appName, L["Fortress"])
	self:RegisterChatCommand("fortress", ChatCmd)
	self:RegisterChatCommand("ft", ChatCmd)
end

function Fortress:AddObjectOptions(name)
	local t = self.optionsFrames[name] or {}
	t.name = name
	t.desc = L["Options for %s"]:format(name)
	t.type = "group"
	t.args = pluginOptions
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(name, t)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(name, name, "Fortress")
end
